#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <time.h>
#include <math.h>

#include <TimeCal_type.h>

#include "ti2utc.h"

#ifndef	TRUE
#define	TRUE	1
#endif
#ifndef	FALSE
#define	FALSE	0
#endif

#define	MAX_TXT_LEN	0x200

int setCalFile(char *timcalfile, char *leapsecfile);

int main(int argc, char **argv)
{
	char   txt[MAX_TXT_LEN];
	int    i, ii, ret;
	int    usec;
	unsigned int setti, eti_ext;

	double elapsed_seconds;

	time_t t_base, t_rough;
	struct tm stm_base, stm_rough, stm_procd;


	for(i = strlen(argv[0]) - 1; i >= 0; i--){
		if(argv[0][i] == '/' || argv[0][i] == '\\'){
			break;
		}
	}
	i++;

	if(argc != 2){
		fprintf(stderr, "TI to UTC converter (ERG)\n");
		fprintf(stderr, "Usage: %s TI\n", &argv[0][i]);
		fprintf(stderr, "          TI: hexadecimal (unit: 1/64[s])\n");
		exit(1);
	}

	// read timcal and leapsecond files
	if(FALSE != setCalFile(FILENAME_TIMCAL, FILENAME_LEAP)){
		fputs("Can not continue. Terminated.\n", stderr);
		exit(1);
	}


	// declare the TI input type as hexadecimal
	if(argv[1][0] != '0' || (argv[1][1] != 'x' && argv[1][1] != 'X')){
		txt[0] = '0';
		txt[1] = 'x';
		strcpy(&txt[2], argv[1]);
	}
	else {
		strcpy(txt, argv[1]);
	}
	setti = strtol(txt, NULL, 0);
	fprintf(stdout, "TI:  0x%08x\n", setti);



	if(6 != sscanf(BASE_ROUGH_TIME, "%04d%02d%02d%02d%02d%02d",
				&stm_base.tm_year, &stm_base.tm_mon,
				&stm_base.tm_mday, &stm_base.tm_hour,
				&stm_base.tm_min,  &stm_base.tm_sec)){
		fprintf(stderr, "Error: format error (BASE_ROUGH_TIME): %s\n",
					BASE_ROUGH_TIME);
		exit(1);
	}
	stm_base.tm_year -= 1900;
	stm_base.tm_mon  -= 1;
	t_base = mktime(&stm_base);

	t_rough = t_base + ((setti - BASE_ROUGH_TI) >> 6);

	gmtime_r(&t_rough, &stm_rough);
	sprintf(txt, "%04d%02d%02d%02d%02d%02d",
				stm_rough.tm_year + 1900,
				stm_rough.tm_mon  + 1,
				stm_rough.tm_mday,
				stm_rough.tm_hour,
				stm_rough.tm_min,
				stm_rough.tm_sec);

	ret = ti2es(txt, setti, &elapsed_seconds, &eti_ext, 55);
	if(ret == RET_AFTER_LAST_RECORD){
		fprintf(stderr, "Warning:: setTI time is later than the last record\n");
	}
	else if(ret == RET_BEFORE_FIRST_RECORD){
		fprintf(stderr, "Warning:: setTI time is earlier than the first record\n");
	}
	else if(ret != RET_NORMAL){
		fprintf(stderr, "Error: ti2es (%d)\n", ret);
		exit(1);
	}


	ret = es2utc(1, elapsed_seconds, txt);
	if(ret != RET_NORMAL){
		fprintf(stderr, "Error: es2utc (%d)\n", ret);
		exit(1);
	}

	if(7 != sscanf(txt, "%04d%02d%02d%02d%02d%02d%06d",
			&stm_procd.tm_year,
			&stm_procd.tm_mon,
			&stm_procd.tm_mday,
			&stm_procd.tm_hour,
			&stm_procd.tm_min,
			&stm_procd.tm_sec,
			&usec) ){
		fprintf(stderr, "Error: decode of es2utc() calc result: %s\n",
					txt);
		exit(1);
	}

	stm_procd.tm_year -= 1900;
	stm_procd.tm_mon  -= 1;

	fprintf(stdout, "UTC: %04d/%02d/%02d %02d:%02d:%02d.%06d\n",
				stm_procd.tm_year + 1900,
				stm_procd.tm_mon  + 1,
				stm_procd.tm_mday,
				stm_procd.tm_hour,
				stm_procd.tm_min,
				stm_procd.tm_sec,
				usec);
}



int setCalFile(char *timcalfile, char *leapsecfile)
{
	int ret;
	char errflg = FALSE;

	if(0 != (ret = setTimeCalFile(timcalfile))){
		errflg = TRUE;
		fprintf(stderr, "Error: timecalfile path set error (%d)\n", ret);
		fprintf(stderr, "       file: %s\n", timcalfile);
	}
	else {
//		fprintf(stderr, "TIMCAL file:     %s\n", timcalfile);
	}

	if(0 != (ret = setLeapSecFile(leapsecfile))){
		errflg = TRUE;
		fprintf(stderr, "Error: leapsecfile path set error (%d)\n", ret);
		fprintf(stderr, "       file: %s\n", leapsecfile);
	}
	else {
//		fprintf(stderr, "LeapSecond file: %s\n", leapsecfile);
	}

	return errflg;
}

